// Import necessary functions from the D3 library
import {
  select,
  csv,
  scaleLinear,
  extent,
  scaleBand,
  axisLeft,
  axisBottom,
  scaleTime,
  line,
  curveBasis,
} from 'd3';
// Select the SVG container in the DOM
const svg = select('svg');
// Extract the width and height of the SVG container from its attributes
const width = +svg.attr('width');
const height = +svg.attr('height');
// Main rendering function for the data visualization
const render = (data) => {
  // Accessor functions for the data's x and y values
  const xValue = (d) => d.day;
  const xAxisLabel = 'Day';

  const yValue = (d) => d.temperature;
  const yAxisLabel = 'Temperature';

  const title = 'Chicago March Average Temperature';
  // Margins around the chart area to place axes labels and title
  const margin = {
    top: 60,
    bottom: 70,
    right: 20,
    left: 100,
  };
  const circleRadius = 8; // Radius for the data point circles (not used in this code)
  // Calculating the inner dimensions of the chart area
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;

  const xScale = scaleTime()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue)) // Extent finds the min and max values in the data for the y-axis
    .range([innerHeight, 0])
    .nice();

  // SVG GROUP
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  g.append('text')
    .attr('y', -10)
    .attr('class', 'title')
    .text(title);

  const lineGenerator = line()
    .x((d) => xScale(xValue(d)))
    .y((d) => yScale(yValue(d)))
    .curve(curveBasis); // Sets the type of curve used to interpolate between points

  g.append('path')
    .attr('class', 'line-path')
    .attr('d', lineGenerator(data));

  // X AXIS
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(20);

  const xAxisG = g
    .append('g')
    .call(xAxis)
    .attr('transform', `translate(0, ${innerHeight})`);

  xAxisG.select('.domain').remove();

  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('x', innerWidth / 2)
    .attr('y', 65)
    .attr('fill', 'black')
    .text(xAxisLabel);

  // Y AXIS
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10);

  const yAxisG = g.append('g').call(yAxis);

  yAxisG.selectAll('.domain').remove();

  yAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('x', -innerHeight / 2)
    .attr('y', -50)
    .attr('fill', 'black')
    .attr('transform', `rotate(-90)`)
    .attr('text-anchor', 'middle')
    .text(yAxisLabel);
};

csv('data.csv').then((data) => {
  // Loading and processing the data
  data.forEach((d) => {
    d.day = new Date(d.day);
    d.temperature = +d.temperature;
  });
  console.log(data);
  render(data);
});
